@@ -12,6 +12,13 @@ from catswalk.scraping.types.type_webdriver import EXECUTION_ENV, DEVICE, DEVICE
 import time
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.keys import Keys
+from PIL import Image
+from io import BytesIO
+from catswalk.scraping.driverfunc.wait import wait_until_images_loaded
+from catswalk.scraping.driverfunc.jquery import import_jquer
+
+
+logger = logging.getLogger()
 
 class CWWebDriver:
     def __init__(self, binary_location: str = None, executable_path: str = None, execution_env: EXECUTION_ENV = EXECUTION_ENV.LOCAL, device = DEVICE.DESKTOP_GENERAL, proxy: str = None, implicitly_wait = 5.0, debug:bool = False):
@@ -29,6 +36,7 @@ class CWWebDriver:
         self.proxy = proxy
         self.device = device
         self.debug = debug
+        self.scrolled = False
 
         options = Options()
         if self.execution_env == EXECUTION_ENV.LOCAL_HEADLESS:
@@ -74,7 +82,6 @@ class CWWebDriver:
         self.driver.implicitly_wait(implicitly_wait)
 
 
-
     def __enter__(self):
         """
 
@@ -143,7 +150,7 @@ class CWWebDriver:
         """
         print(f"wait_rendering_by_class: {_class}")
         if by_css_selector:
-            WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.XPATH, _class)))
+            WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.CSS_SELECTOR, _class)))
         else:
             WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.CLASS_NAME, _class)))
 
@@ -154,6 +161,7 @@ class CWWebDriver:
             url (str): [description]
         """
         self.driver.get(url)
+        self.scrolled = False
 
     def get(self, url: str):
         self.transition(url=url)
@@ -177,6 +185,28 @@ class CWWebDriver:
             f.write(png)
         return fullpath
 
+
+    def print_screen_by_size(self, w, h, path, filename, start_w: int = 0, start_h: int = 0):
+        """
+        hw = self.driver.get_window_size()
+        w = hw["width"]
+        h = hw["height"]
+        """
+        img_binary = self.driver.get_screenshot_as_png()
+        img = Image.open(BytesIO(img_binary))
+        print(f"print_screen_by_size, {start_w}, {start_h}, {w}, {h}")
+        crop_dim = (start_w, start_h, w, h) # left top right bottom
+        img = img.crop(crop_dim)
+        fullpath = f"{path}/{filename}.png"
+        img.save(fullpath)
+        return fullpath
+
+    def print_screen_by_hight(self, h, path, filename, scale: int = 1):
+        hw = self.driver.get_window_size()
+        w = hw["width"] * scale
+        fullpath = self.print_screen_by_size(w=int(w), h=int(h), path=path, filename=filename)
+        return fullpath
+
     def print_screen_by_window(self, path, filename):
         """[summary]
 
@@ -187,27 +217,28 @@ class CWWebDriver:
         Returns:
             [type]: [description]
         """
-        # get width and height of the page
-        hw = self.driver.get_window_size()
-        w = hw["width"]
-        h = hw["height"]
-
         # Get Screen Shot
         fullpath = f"{path}/{filename}.png"
+        wait_until_images_loaded(driver=self.driver)
+        if self.position_height != 0:
+            self.scroll_by_offset(offset=1)
+            self.scroll_by_offset(offset=-1)
+            self.scroll_by_offset(offset=-1)
+            self.scroll_by_offset(offset=1)
         self.driver.save_screenshot(fullpath)
         return fullpath
 
-    def __get_elem_by_class(self, class_name:str):
+    def get_elem_by_class(self, class_name:str):
         if len(class_name.split(" ")) > 1:
             _class_name = "." + ".".join(class_name.split(" "))
-            #self.wait_rendering_by_class(_class_name, True)
+            self.wait_rendering_by_class(_class_name, True)
             elem = self.driver.find_element_by_css_selector(_class_name)
         else:
             self.wait_rendering_by_class(class_name, False)
             elem = self.driver.find_element_by_class_name(class_name)
         return elem
 
-    def click_by_class_name(self, class_name:str) -> str:
+    def click_by_class_name(self, class_name:str, check_exist:bool = False) -> str:
         """[summary]
 
         Args:
@@ -217,7 +248,10 @@ class CWWebDriver:
             str: [description]
         """
         # Get Screen Shot
-        elem = self.__get_elem_by_class(class_name)
+        if check_exist:
+            if not self.is_exist_class(class_name=class_name):
+                return f"{class_name} not found"
+        elem = self.get_elem_by_class(class_name)
         elem.click()
         if self.debug:
             time.sleep(5)
@@ -234,6 +268,24 @@ class CWWebDriver:
         for num in range(0, scroll_time):
             element.send_keys(Keys.PAGE_DOWN)
 
+    def smooth_scroll_to_bottom(self, scroll_pause_time = 0.5):
+        # lazy対応
+        # https://stackoverflow.com/questions/62600288/how-to-handle-lazy-loaded-images-in-selenium
+        print("smooth_scroll_to_bottom")
+        scroll_pause_time = 2.0
+        i = 0
+        last_height = self.driver.execute_script("return document.body.scrollHeight")
+        while True:
+            self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
+            time.sleep(scroll_pause_time)
+            new_height = self.driver.execute_script("return document.body.scrollHeight")
+            if new_height == last_height:
+                break
+            last_height = new_height
+            i += 1
+            if i == 5:
+                break
+
     def move_to_element_by_class_name(self, class_name:str) -> str:
         """[summary]
 
@@ -243,11 +295,38 @@ class CWWebDriver:
         Returns:
             str: [description]
         """
-        element = self.__get_elem_by_class(class_name=class_name)
+        if not self.scrolled:
+            self.smooth_scroll_to_bottom()
+        element = self.get_elem_by_class(class_name=class_name)
         element.location_once_scrolled_into_view
+        self.scrolled = True
         if self.debug:
             time.sleep(5)
 
+    def is_exist_class(self, class_name: str) -> bool:
+        """[summary]
+
+        Args:
+            class_name (str): [description]
+
+        Returns:
+            bool: [description]
+        """
+        html = self.html
+        _class_name = "." + ".".join(class_name.split(" "))
+        if html.select(_class_name):
+            logger.debug(f"is_exist_class, {class_name} is found")
+            return True
+        else:
+            logger.debug(f"is_exist_class, {class_name} not found")
+            return False
+        
+    @property
+    def position_height(self):
+        hight = self.driver.execute_script("return document.body.scrollHeight")
+        print(f"position_height: {hight}")
+        return hight
+
     @property
     def html(self):
         html = self.driver.page_source.encode('utf-8')
